import functools
import datetime
from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for, current_app
)
from werkzeug.security import check_password_hash, generate_password_hash
from wtforms.validators import StopValidation, ValidationError
from flaskr.db import get_db, UserTask, PostQ_ham, PostQ_ham_2, UserBehavior, PostQ_ham_3
from flaskr.template import OptionRequired
from flaskr.util import return_img_stream
from sqlalchemy.sql import text
from sqlalchemy import update
import os
import pandas as pd
import numpy as np
import json
ham_bp = Blueprint('ham_medical', __name__, url_prefix='/ham_medical')

from flask_wtf import FlaskForm
from wtforms import widgets, StringField, SubmitField, SelectField, RadioField, SelectMultipleField, TextAreaField, IntegerRangeField
from wtforms.validators import DataRequired, Length, InputRequired, NumberRange
from markupsafe import Markup


success_link = "https://app.prolific.co/submissions/complete?cc=COQI0CT2"
failure_link = "https://app.prolific.co/submissions/complete?cc=C1EU101H"
 
class BootstrapListWidget(widgets.ListWidget):
 
    def __call__(self, field, **kwargs):
        kwargs.setdefault("id", field.id)
        # html = [f"<{self.html_tag} {widgets.html_params(**kwargs)}>"]
        html = [f"<ul class='list-group-flash border-0'>"]
        max_index = 0
        for index, subfield in enumerate(field):
            if index % 3 == 0:
                html.append(f"<div class='row'>")
            if self.prefix_label:
                html.append(f"<div class='list-group-item col-sm-4 border-0'>{subfield.label} {subfield(class_='form-check-input ms-1')}</div>\n")
            else:
                html.append(f"<div class='list-group-item col-sm-4 border-0'>{subfield(class_='form-check-input me-1')} {subfield.label}</div>\n")
            if index % 3 == 2:
                html.append(f"</div>")
            max_index = index
        if max_index % 3 != 2:
            html.append(f"</div>")
        html.append("</%s>" % self.html_tag)
        return Markup("".join(html))
 
 
class MultiCheckboxField(SelectMultipleField):
    """
    A multiple-select, except displays a list of checkboxes.
 
    Iterating the field will produce subfields, allowing custom rendering of
    the enclosed checkbox fields.
    """
    widget = BootstrapListWidget(prefix_label=False)
    option_widget = widgets.CheckboxInput()

class MultiCheckboxAtLeastOne(object):
    def __init__(self, message=None):
        if not message:
            message = 'At least one option must be selected.'
        self.message = message

    def __call__(self, form, field):
        if len(field.data) == 0:
            flash(self.message)
            raise StopValidation(self.message)

class NoConflictChoice(object):
    """
    Checks the field's data is 'truthy' otherwise stops the validation chain.

    This validator checks that the ``data`` attribute on the field is a 'true'
    value (effectively, it does ``if field.data``.) Furthermore, if the data
    is a string type, a string containing only whitespace characters is
    considered false.

    If the data is empty, also removes prior errors (such as processing errors)
    from the field.

    **NOTE** this validator used to be called `Required` but the way it behaved
    (requiring coerced data, not input data) meant it functioned in a way
    which was not symmetric to the `Optional` validator and furthermore caused
    confusion with certain fields which coerced data to 'falsey' values like
    ``0``, ``Decimal(0)``, ``time(0)`` etc. Unless a very specific reason
    exists, we recommend using the :class:`InputRequired` instead.

    :param message:
        Error message to raise in case of a validation error.
    """

    def __init__(self, message=None):
        self.message = message

    def __call__(self, form, field):
        # print(field.data)
        list1 = form.malignant_features.data
        list2 = form.benign_features.data
        set1 = set(list1)
        set2 = set(list2)
        if len(set1 & set2) > 0:
            if self.message is None:
                message = field.gettext('One concept will not contribute to both malignant and benign.')
            else:
                message = self.message
            flash(message)
            field.errors[:] = []
            raise ValidationError(message)

class HamDataset(object):

    def __init__(self, tp_task):
        self.task_id = tp_task['image_id']
        self.image_path = tp_task['path']
        self.cav = tp_task['cav']
        self.score = tp_task['score']
        self.ground_truth = tp_task['ground_truth']
        self.prediction = tp_task['advice']
        self.correctness = tp_task['correctness']
        self.concept_explanation = tp_task['concept_explanation']
        # dict, key: [positive_claim, negative_claim]. value can be null/None
        self.archie_explanation = tp_task['archie_explanation']
        # dict, key: [positive_claim, negative_claim], value list, can be empty list

    @staticmethod
    def deal_int(value):
        if value is not None:
            out_str = "%d"%(value)
        else:
            out_str = "0"
        return out_str

    def print_generation_task(self):
        task_str = ""
        task_str += "Task %s"%(self.task_id)
        task_str += ", with label %s"%(self.prediction)
        task_str += ", with ground truth %s"%(self.ground_truth)
        return task_str

    def serize(self):
        return {
            "task_id": self.task_id,
            "image_path": self.image_path,
            "cav": self.cav,
            "score": self.score,
            "ground_truth": self.ground_truth,
            "prediction": self.prediction,
            "concept_explanation": self.concept_explanation,
            "archie_explanation": self.archie_explanation
        }

def load_generation_tasks(filename="flaskr/static/data/ham_medical/selected_tasks_new.json"):
    f = open(filename)
    data_object = json.load(f)
    task_list = data_object['task_list']
    model_parameter = data_object['model_parameter']
    f.close()
    data_dict = {}
    task_ID_list = []
    for i in range(len(task_list)):
        tp_data = HamDataset(task_list[i])
        data_dict[tp_data.task_id] = tp_data
        task_ID_list.append(tp_data.task_id)
    # print(data_dict['T003'].print_generation_task())
    return task_ID_list, data_dict

task_ID_list, ham_task_dict = load_generation_tasks("flaskr/static/data/ham_medical/selected_tasks_final.json")

# concept_list = ['BWV', 'RegularDG', 'IrregularDG', 'RegressionStructures', 'IrregularStreaks', 'RegularStreaks', 'AtypicalPigmentNetwork', 'TypicalPigmentNetwork']
# concept_name_dict = {
#     'BMW': "Blue-Whitish Veil",
#     "RegularDG": "Regular Dots & Globules",
#     "IrregularDG": "Irregular Dots & Globules",
#     "RegressionStructures": "Regression Structures",
#     "IrregularStreaks": "Irregular Streaks",
#     "RegularStreaks": "Regular Streaks",
#     "AtypicalPigmentNetwork": "Atypical Pigment Network",
#     "TypicalPigmentNetwork": "Typical Pigment Network"
# }

class submit_form(FlaskForm):
    submit_button = SubmitField('Next')


@ham_bp.route('/instruction', methods=('GET', 'POST'))
def instruction():
    # show the user instruction for whole task
    # form = submit_form()
    # image_path = "flaskr/static/images/bird/lesser_goldfinch_1.jpg"
    # img_stream = return_img_stream(image_path)
    condition = session['condition']
    if request.method == 'POST':
        if condition == "control" or condition == "pilot":
            # in control condition, no explanations are provided, we also do not show the concepts to users
            return redirect(url_for("ham_medical.example"))
        else:
            return redirect(url_for("ham_medical.concepts"))
    return render_template('ham_medical/instruction.html', condition=condition)

# ignored, we use two examples to introduce the task
# @bp.route('/task_intro', methods=['GET', 'POST'])
# def task_intro():
#     if request.method == 'POST':
#         return redirect(url_for("ham_medical.example"))
#     condition = session['condition']
#     return render_template('ham_medical/task_intro.html', condition=condition)

@ham_bp.route('/concepts', methods=['GET', 'POST'])
def concepts():
    if request.method == 'POST':
        return redirect(url_for("ham_medical.example"))
    image_path = "flaskr/static/images/ham_medical/tutorial_1.jpg"
    # img_stream = return_img_stream(image_path)
    return render_template('ham_medical/concepts.html')

@ham_bp.route('/example', methods=['GET', 'POST'])
def example():
    if request.method == 'POST':
        return redirect(url_for("ham_medical.example2"))
    image_path = "flaskr/static/images/ham_medical/ISIC_0028155.jpg"
    # malignant example
    # print(os.path.isfile(image_path))
    # img_stream = return_img_stream(image_path)
    condition = session['condition']
    return render_template('ham_medical/example.html', condition=condition)

@ham_bp.route('/example2', methods=['GET', 'POST'])
def example2():
    if request.method == 'POST':
        return redirect(url_for("ham_medical.expertise"))
    image_path = "flaskr/static/images/ham_medical/ISIC_0031325.jpg"
    # benign example
    # print(os.path.isfile(image_path))
    # img_stream = return_img_stream(image_path)
    condition = session['condition']
    return render_template('ham_medical/example_2.html', condition=condition)

# expertise_choice = [('1', 'no prior experience or relevant knowledge' ), 
#                     ('2', '1-5 years of learning experience on medical topics'),
#                     ('3', 'more than 5 years of learning experience'),
#                     ('4', 'more than 5 years of learning experience and practical medical diagnosis experience')]
# skin_cancer_expertise_choice = [('1', 'I know nothing about skin cancer'),
#                                 ('2', 'I roughly know some medical concepts about skin cancer, but can not diagnose it'),
#                                 ('3', 'I understand it quite well, and am able to diagnose it')]
Likert_list = [('1', '1: No prior experience/knowledge'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5: Extensive prior experience/knowledge')]

class ExpertiseForm(FlaskForm):
    medical_expertise = RadioField('To what extent are you knowledgeable about medical diagnosis?', choices=Likert_list, validators=[DataRequired()])
    skin_cancer_expertise = RadioField('Do you have any experience or knowledge about skin cancer?', choices=Likert_list, validators=[DataRequired()])
    submit_button = SubmitField('Submit')

def save_expertise(user_id, event_desc, user_behavior):
    db = get_db()
    tp_obj = UserBehavior(user_id=user_id, event_desc=event_desc, user_behavior=user_behavior)
    q = db.query(UserBehavior.user_id).filter(UserBehavior.user_id==user_id, UserBehavior.event_desc==event_desc)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        db.add(tp_obj)
        db.commit()

@ham_bp.route('/expertise', methods=['GET', 'POST'])
def expertise():
    form = ExpertiseForm()
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        medical_expertise = form.medical_expertise.data
        skin_cancer_expertise = form.skin_cancer_expertise.data
        save_expertise(user_id, "medical_expertise", medical_expertise)
        save_expertise(user_id, "skin_cancer_expertise", skin_cancer_expertise)
        return redirect(url_for("ham_medical.ATI_preq"))
    image_path = "flaskr/static/images/ham_medical/ISIC_0031325.jpg"
    # benign example
    # print(os.path.isfile(image_path))
    # img_stream = return_img_stream(image_path)
    # condition = session['condition']
    return render_template('ham_medical/expertise.html', form=form)


def save_user_info(user_id, user_task_order, user_group):
    from flaskr.db import UserInfo
    db = get_db()
    q = db.query(UserInfo.user_id).filter(UserInfo.user_id==user_id)
    exist_flag = db.query(q.exists()).scalar()
    # if exist_flag:
    #     return True
    task_order_str = "|".join(user_task_order)
    reverse_flag_str = "None"
    analogy_type = "None"

    if not exist_flag:
        ui_obj = UserInfo(user_id=user_id, task_order_str=task_order_str, reverse_flag_str=reverse_flag_str, user_group=user_group, analogy_type=analogy_type)
        db.add(ui_obj)
        db.commit()
    else:
        stmt = update(UserInfo).where(UserInfo.user_id==user_id).values(task_order_str=task_order_str, user_group=user_group).execution_options(synchronize_session="fetch")
        result = db.execute(stmt)
        db.commit()
        current_app.logger.info("User {} exist, update userinfo".format(user_id))

    return False

@ham_bp.route('/user_port', methods=['GET', 'POST'])
def user_port():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    session['attention_chck_fail'] = 0
    # exp_conditions = ["control", "ARCH", "ARCHIE"]
    # condition = np.random.choice(exp_conditions, 1)[0]
    session['condition'] = "pilot"
    
    tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = tp_task_ID_list
    session['check_pass'] = False
    session['check_pass_2'] = False
    current_app.logger.info("User {} log in, with condition pilot".format(user_id))
    exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, user_group="pilot")
    return redirect(url_for("ham_medical.instruction"))

@ham_bp.route('/user_port_1', methods=['GET', 'POST'])
def user_port_1():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    session['attention_chck_fail'] = 0
    # exp_conditions = ["control", "ARCH", "ARCHIE"]
    # condition = np.random.choice(exp_conditions, 1)[0]
    session['condition'] = "control"
    
    tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = tp_task_ID_list
    session['check_pass'] = False
    session['check_pass_2'] = False
    current_app.logger.info("User {} log in, with condition control".format(user_id))
    exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, user_group="control")
    return redirect(url_for("ham_medical.instruction"))


@ham_bp.route('/user_port_2', methods=['GET', 'POST'])
def user_port_2():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    session['attention_chck_fail'] = 0
    # exp_conditions = ["control", "ARCH", "ARCHIE"]
    # condition = np.random.choice(exp_conditions, 1)[0]
    session['condition'] = "concept"
    
    tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = tp_task_ID_list
    session['check_pass'] = False
    session['check_pass_2'] = False
    current_app.logger.info("User {} log in, with condition concept".format(user_id))
    exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, user_group="concept")
    return redirect(url_for("ham_medical.instruction"))

@ham_bp.route('/user_port_3', methods=['GET', 'POST'])
def user_port_3():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    session['attention_chck_fail'] = 0
    # exp_conditions = ["control", "ARCH", "ARCHIE"]
    # condition = np.random.choice(exp_conditions, 1)[0]
    session['condition'] = "archie_target"
    
    tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = tp_task_ID_list
    session['check_pass'] = False
    session['check_pass_2'] = False
    current_app.logger.info("User {} log in, with condition ARCHIE-target".format(user_id))
    exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, user_group="archie_target")
    return redirect(url_for("ham_medical.instruction"))

@ham_bp.route('/user_port_4', methods=['GET', 'POST'])
def user_port_4():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    session['attention_chck_fail'] = 0
    # exp_conditions = ["control", "ARCH", "ARCHIE"]
    # condition = np.random.choice(exp_conditions, 1)[0]
    session['condition'] = "archie"
    
    tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = tp_task_ID_list
    session['check_pass'] = False
    session['check_pass_2'] = False
    current_app.logger.info("User {} log in, with condition ARCHIE".format(user_id))
    exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, user_group="archie")
    return redirect(url_for("ham_medical.instruction"))

@ham_bp.route('/user_port_5', methods=['GET', 'POST'])
def user_port_5():
    user_id = request.args.get('PROLIFIC_PID')
    # To-do: recognize participated users, and ask them to leave
    # we may record the time user enter our web app
    # show the user profile for that user
    session['user_id'] = user_id
    session['task_id'] = 0
    session['attention_chck_right'] = 0
    session['attention_chck_fail'] = 0
    # exp_conditions = ["control", "ARCH", "ARCHIE"]
    # condition = np.random.choice(exp_conditions, 1)[0]
    session['condition'] = "archie_od"
    
    tp_task_ID_list = np.random.permutation(task_ID_list).tolist()
    session['task_ID_list'] = tp_task_ID_list
    session['check_pass'] = False
    session['check_pass_2'] = False
    current_app.logger.info("User {} log in, with condition ARCHIE".format(user_id))
    exist_flag = save_user_info(user_id=user_id, user_task_order=tp_task_ID_list, user_group="archie_od")
    return redirect(url_for("ham_medical.instruction"))

def save_annotation(user_id, task_id, answer_type, user_choice):
    db = get_db()
    error = None
    q = db.query(UserTask.user_id).filter(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        ut_obj = UserTask(user_id=user_id, task_id=task_id, answer_type=answer_type, choice=user_choice)
        db.add(ut_obj)
        db.commit()

ham_choices = [('benign', 'benign: not harmful in effect'), ('malignant', 'malignant: harmful in effect')]
experience_choices = [('No', 'No, I had never seen it before'), ('Yes', 'Yes, I saw it on myself or others')]
confidence_choices = [('unconfident', 'unconfident'), ('somewhat unconfident', 'somewhat unconfident'), ('neutral', 'neutral'),
                        ('somewhat confident', 'somewhat confident'), ('confident', 'confident')]
class HamForm(FlaskForm):
    question = RadioField('Given the image, is it malignant or benign?', choices=ham_choices, validators=[DataRequired()], render_kw={"style": "font-weight: bold;"})
    experience = RadioField('Have you ever had this or seen it on others ?', choices=experience_choices, validators=[DataRequired()])
    confidence = RadioField('How confident are you with your decision?', choices=confidence_choices, validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class HamForm_2(FlaskForm):
    question = RadioField('Given the image, is it malignant or benign?', choices=ham_choices, validators=[DataRequired()], render_kw={"style": "font-weight: bold;"})
    confidence = RadioField('How confident are you with your decision?', choices=confidence_choices, validators=[DataRequired()])
    submit_button = SubmitField('Submit')


@ham_bp.route('/ham_task_pilot', methods=['GET', 'POST'])
def ham_task_pilot():
    form = HamForm()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data
        user_experience = form.experience.data
        user_confidence = form.confidence.data
        # Shall we only ask it once at the second stage of the task?
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="pilot", user_choice=user_choice)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="pilot_experience", user_choice=user_experience)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="pilot_confidence", user_choice=user_confidence)
        # if task_id < len(task_ID_list) -1:
        # start next task
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('ham_medical.ham_task_pilot'))
        else:
            # return redirect(success_link)
            return redirect(url_for('ham_medical.postq_3'))
    # To check further, whether we need attention check here
    if task_id == 5 and not session['check_pass']:
        # At task 6, come to attention check
        response = redirect(url_for('ham_medical.attention_check'))
        return response
    # if task_id == 10 and not session['check_pass_2']:
    #     # At task 11, come to attention check 2
    #     response = redirect(url_for('ham_medical.attention_check_2'))
    #     return response
    tp_data = ham_task_dict[tp_task_ID].serize()
    image_path = tp_data['image_path']
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    return render_template('ham_medical/ham_task.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice="None", explanation_sent="None")


# first stage of decision making
@ham_bp.route('/ham_task_base', methods=['GET', 'POST'])
def ham_task_base():
    form = HamForm()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        condition = session['condition']
        user_choice = form.question.data
        user_experience = form.experience.data
        user_confidence = form.confidence.data
        # Shall we only ask it once at the second stage of the task?
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="base", user_choice=user_choice)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="base_experience", user_choice=user_experience)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="base_confidence", user_choice=user_confidence)
        # if task_id < len(task_ID_list) -1:
        # start second stage
        if condition == "archie":
            return redirect(url_for('ham_medical.ham_task_archie'))
        elif condition == "archie_target":
            return redirect(url_for('ham_medical.ham_task_archie_t'))
        elif condition == "archie_od":
            return redirect(url_for('ham_medical.ham_task_archie_o'))
        elif condition == "concept":
            return redirect(url_for('ham_medical.ham_task_concept'))
        else:
            return redirect(url_for('ham_medical.ham_task_control'))
    # To check further, whether we need attention check here
    if task_id == 5 and not session['check_pass']:
        # At task 6, come to attention check
        response = redirect(url_for('ham_medical.attention_check'))
        return response
    # if task_id == 10 and not session['check_pass_2']:
    #     # At task 11, come to attention check 2
    #     response = redirect(url_for('ham_medical.attention_check_2'))
    #     return response
    tp_data = ham_task_dict[tp_task_ID].serize()
    image_path = tp_data['image_path']
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    return render_template('ham_medical/ham_task.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice="None", explanation_sent="None")

class HamAttentionForm_1(FlaskForm):
    question = RadioField('Please select benign here', choices=ham_choices, validators=[DataRequired()])
    experience = RadioField('Select Yes here', choices=experience_choices, validators=[DataRequired()])
    confidence = RadioField('Select Neutral here', choices=confidence_choices, validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class HamAttentionForm_2(FlaskForm):
    question = RadioField('Please select malignant here', choices=ham_choices, validators=[DataRequired()])
    experience = RadioField('Select No here', choices=experience_choices, validators=[DataRequired()])
    confidence = RadioField('Select somewhat confident here', choices=confidence_choices, validators=[DataRequired()])
    submit_button = SubmitField('Submit')

@ham_bp.route('/attention_check', methods=('GET', 'POST'))
def attention_check():
    form = HamAttentionForm_1()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST':
        session['check_pass'] = True
        user_id = session['user_id']
        user_choice = form.question.data
        user_experience = form.experience.data
        user_confidence = form.confidence.data
        save_choice(user_id, 'attention_ham_1', user_choice, answer_type="choice")
        save_choice(user_id, 'attention_ham_1', user_experience, answer_type="experience")
        save_choice(user_id, 'attention_ham_1', user_confidence, answer_type="confidence")
        if user_choice == "benign" and user_experience == "Yes" and user_confidence == "neutral":
            session['attention_chck_right'] += 1
        else:
            session['attention_chck_fail'] += 1
            if session['attention_chck_fail'] >= 2:
                return redirect(failure_link)
        # leave_time = datetime.datetime.utcnow()
        user_condition = session["condition"]
        if user_condition == "pilot":
            return redirect(url_for('ham_medical.ham_task_pilot'))
        return redirect(url_for('ham_medical.ham_task_base'))
        # return redirect(url_for('loan_analogy.base'))
    tp_data = ham_task_dict[tp_task_ID].serize()
    image_path = tp_data['image_path']
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    return render_template('ham_medical/attention_check.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice="None", explanation_sent="None")

@ham_bp.route('/attention_check_2', methods=('GET', 'POST'))
def attention_check_2():
    form = HamAttentionForm_2()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST':
        session['check_pass_2'] = True
        user_id = session['user_id']
        user_choice = form.question.data
        user_experience = form.experience.data
        user_confidence = form.confidence.data
        save_choice(user_id, 'attention_ham_2', user_choice, answer_type="choice")
        save_choice(user_id, 'attention_ham_2', user_experience, answer_type="experience")
        save_choice(user_id, 'attention_ham_2', user_confidence, answer_type="confidence")
        if user_choice == "malignant" and user_experience == "No" and user_confidence == "somewhat confident":
            session['attention_chck_right'] += 1
        else:
            session['attention_chck_fail'] += 1
        if session['attention_chck_fail'] >= 2:
            return redirect(failure_link)
        # leave_time = datetime.datetime.utcnow()
        user_condition = session["condition"]
        if user_condition == "pilot":
            return redirect(url_for('ham_medical.ham_task_pilot'))
        return redirect(url_for('ham_medical.ham_task_base'))
        # return redirect(url_for('loan_analogy.base'))
    tp_data = ham_task_dict[tp_task_ID].serize()
    image_path = tp_data['image_path']
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    return render_template('ham_medical/attention_check.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice="None", explanation_sent="None")

# condition control, second stage of decision making
@ham_bp.route('/ham_task_control', methods=['GET', 'POST'])
def ham_task_control():
    form = HamForm_2()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data
        user_confidence = form.confidence.data
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="control", user_choice=user_choice)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="control_confidence", user_choice=user_confidence)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('ham_medical.ham_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return redirect(url_for('ham_medical.nasatlx'))
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    tp_data = ham_task_dict[tp_task_ID].serize()
    image_path = tp_data['image_path']
    prediction = tp_data['prediction']
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    return render_template('ham_medical/ham_control.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice=prediction, explanation_sent="None")

# condition concept, second stage of decision making
@ham_bp.route('/ham_task_concept', methods=['GET', 'POST'])
def ham_task_concept():
    form = HamForm_2()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data
        user_confidence = form.confidence.data
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="concept", user_choice=user_choice)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="concept_confidence", user_choice=user_confidence)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('ham_medical.ham_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return redirect(url_for('ham_medical.nasatlx'))
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    tp_data = ham_task_dict[tp_task_ID].serize()
    positive_claim = tp_data['concept_explanation']['positive_claim']
    negative_claim = tp_data['concept_explanation']['negative_claim']
    image_path = tp_data['image_path']
    prediction = tp_data['prediction']
    # explanation_sent = f"Given this image, AI system observed following concepts: [<strong>{concepts}</strong>]. And AI system provides advice: [<strong>{prediction}</strong>]."
    # explanation_sent = "Model found xxx, and yyy absent in this image"
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    return render_template('ham_medical/ham_concept.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice=prediction, positive_claim=positive_claim, negative_claim=negative_claim)

@ham_bp.route('/useanalogy', methods=['POST'])
def useanalogy():
    db = get_db()
    user_id = session['user_id']
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    data = request.get_json()
    name = data['name']
    event_desc = tp_task_ID + "|" + name
    user_behavior = "click"
    q = db.query(UserBehavior.user_id).filter(UserBehavior.user_id==user_id, UserBehavior.event_desc==event_desc)
    exist_flag = db.query(q.exists()).scalar()
    # current_app.logger.info("User {} clicked this button, existence: {}".format(user_id, exist_flag))
    if not exist_flag:
        ut_obj = UserBehavior(user_id=user_id, event_desc=event_desc, user_behavior=user_behavior)
        db.add(ut_obj)
        db.commit()
    return ('', 204)

def archie_explanation_gen(claim_list):
    if len(claim_list) == 0:
        claim_string = "None"
    elif len(claim_list) == 1:
        concept = claim_list[0]['concept']
        explanation = claim_list[0]['explanation']
        claim_string = f"<strong>{concept}</strong>: {explanation}"
    else:
        claim_string = "<ul>"
        for claim in claim_list:
            concept = claim['concept']
            explanation = claim['explanation']
            claim_string += f"<li> <strong>{concept}</strong>: {explanation} <li/>"
        claim_string += "</ul>"
    return claim_string

# condition archie, second stage of decision making
@ham_bp.route('/ham_task_archie', methods=['GET', 'POST'])
def ham_task_archie():
    form = HamForm_2()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data
        user_confidence = form.confidence.data
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="archie", user_choice=user_choice)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="archie_confidence", user_choice=user_confidence)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('ham_medical.ham_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return redirect(url_for('ham_medical.nasatlx'))
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    tp_data = ham_task_dict[tp_task_ID].serize()
    # positive_claim = archie_explanation_gen(tp_data['archie_explanation']['positive_claim'])
    # negative_claim = archie_explanation_gen(tp_data['archie_explanation']['negative_claim'])
    positive_claim = tp_data['archie_explanation']['positive_claim']
    negative_claim = tp_data['archie_explanation']['negative_claim']
    image_path = tp_data['image_path']
    # concepts = tp_data['concepts']
    prediction = tp_data['prediction']
    # explanation_sent = f"Given this image, AI system observed following concepts: [<strong>{concepts}</strong>]. And AI system provides advice: [<strong>{prediction}</strong>]."
    # explanation_sent = "Model found xxx, and yyy absent in this image. It's like (analogy-based explanation)"
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    # print(positive_claim)
    # print(negative_claim)
    return render_template('ham_medical/ham_archie.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice=prediction, positive_claim=positive_claim, negative_claim=negative_claim)

# condition archie_target, second stage of decision making
@ham_bp.route('/ham_task_archie_t', methods=['GET', 'POST'])
def ham_task_archie_t():
    form = HamForm_2()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data
        user_confidence = form.confidence.data
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="archiet", user_choice=user_choice)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="archiet_confidence", user_choice=user_confidence)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('ham_medical.ham_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return redirect(url_for('ham_medical.nasatlx'))
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    tp_data = ham_task_dict[tp_task_ID].serize()
    # positive_claim = archie_explanation_gen(tp_data['archie_explanation']['positive_claim'])
    # negative_claim = archie_explanation_gen(tp_data['archie_explanation']['negative_claim'])
    positive_claim = tp_data['archie_explanation']['positive_claim']
    negative_claim = tp_data['archie_explanation']['negative_claim']
    image_path = tp_data['image_path']
    # concepts = tp_data['concepts']
    prediction = tp_data['prediction']
    # explanation_sent = f"Given this image, AI system observed following concepts: [<strong>{concepts}</strong>]. And AI system provides advice: [<strong>{prediction}</strong>]."
    # explanation_sent = "Model found xxx, and yyy absent in this image. It's like (analogy-based explanation)"
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    # print(positive_claim)
    # print(negative_claim)
    return render_template('ham_medical/ham_archie_t.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice=prediction, positive_claim=positive_claim, negative_claim=negative_claim)

# condition archie_target, second stage of decision making
@ham_bp.route('/ham_task_archie_o', methods=['GET', 'POST'])
def ham_task_archie_o():
    form = HamForm_2()
    task_id = session['task_id']
    task_ID_list = session['task_ID_list']
    tp_task_ID = task_ID_list[task_id]
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        user_choice = form.question.data
        user_confidence = form.confidence.data
        # phase = "base"
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="archieo", user_choice=user_choice)
        save_annotation(user_id=user_id, task_id=tp_task_ID, answer_type="archieo_confidence", user_choice=user_confidence)
        if task_id < len(task_ID_list) -1:
            session['task_id'] += 1
            # start next task
            return redirect(url_for('ham_medical.ham_task_base'))
        else:
            # return redirect(url_for('analogy_generation.post_questionnaire'))
            return redirect(url_for('ham_medical.nasatlx'))
            # return redirect("https://app.prolific.co/submissions/complete?cc=3E156809")
    tp_data = ham_task_dict[tp_task_ID].serize()
    # positive_claim = archie_explanation_gen(tp_data['archie_explanation']['positive_claim'])
    # negative_claim = archie_explanation_gen(tp_data['archie_explanation']['negative_claim'])
    positive_claim = tp_data['archie_explanation']['positive_claim']
    negative_claim = tp_data['archie_explanation']['negative_claim']
    image_path = tp_data['image_path']
    # concepts = tp_data['concepts']
    prediction = tp_data['prediction']
    # explanation_sent = f"Given this image, AI system observed following concepts: [<strong>{concepts}</strong>]. And AI system provides advice: [<strong>{prediction}</strong>]."
    # explanation_sent = "Model found xxx, and yyy absent in this image. It's like (analogy-based explanation)"
    img_stream = return_img_stream(image_path)
    # img_stream_tutorial = return_img_stream("flaskr/static/images/ham_medical/tutorial_1.jpg")
    # print(positive_claim)
    # print(negative_claim)
    return render_template('ham_medical/ham_archie_od.html', task_index=task_id+1, task_total=len(task_ID_list), form=form, 
        img_stream=img_stream, advice=prediction, positive_claim=positive_claim, negative_claim=negative_claim)

def save_choice(user_id, task_id, choice, answer_type="attention"):
    db = get_db()
    error = None
    from flaskr.db import UserTask
    q = db.query(UserTask.user_id).filter(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        ut_obj = UserTask(user_id=user_id, task_id=task_id, choice=choice, answer_type=answer_type)
        db.add(ut_obj)
        db.commit()
    # else:
    #     stmt = update(UserTask).where(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type).values(choice=choice).execution_options(synchronize_session="fetch")
    #     result = db.execute(stmt)
    #     db.commit()
    #     current_app.logger.info("User {} has record taskid-answer_type:{}-{} exist, update usertask".format(user_id, task_id, answer_type))

ATI_choices = choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')]
class ATIForm(FlaskForm):
    answer_1 = RadioField('I like to occupy myself in greater detail with technical systems.', choices=ATI_choices, validators=[DataRequired()])
    answer_2 = RadioField('I like testing the functions of new technical systems.', choices=ATI_choices, validators=[DataRequired()])
    answer_3 = RadioField('I predominantly deal with technical systems because I have to.', choices=ATI_choices, validators=[DataRequired()])
    answer_4 = RadioField('When I have a new technical system in front of me, I try it out intensively.', choices=ATI_choices, validators=[DataRequired()])
    answer_5 = RadioField('I enjoy spending time becoming acquainted with a new technical system.', choices=ATI_choices, validators=[DataRequired()])
    answer_6 = RadioField('It is enough for me that a technical system works; I don’t care how or why.', choices=ATI_choices, validators=[DataRequired()])
    
    answer_7 = RadioField('I try to understand how a technical system exactly works.', choices=ATI_choices, validators=[DataRequired()])
    answer_8 = RadioField('It is enough for me to know the basic functions of a technical system.', choices=ATI_choices, validators=[DataRequired()])
    answer_9 = RadioField('I try to make full use of the capabilities of a technical system.', choices=ATI_choices, validators=[DataRequired()])
    answer_10 = RadioField('Attention check: choose option completely agree.', choices=ATI_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

@ham_bp.route('/ATI_preq', methods=('GET', 'POST'))
def ATI_preq():
    form = ATIForm()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in pre-questionnaire".format(user_id))
            # answer_1 = form.answer_1.data
            # answer_2 = form.answer_2.data
            # answer_3 = form.answer_3.data
            # answer_4 = form.answer_4.data
            # answer_5 = form.answer_5.data
            # answer_6 = form.answer_6.data
            # answer_7 = form.answer_7.data
            # answer_8 = form.answer_8.data
            # answer_9 = form.answer_9.data
            answer_dict = {"user_id": user_id}
            for i in range(9):
                tp_key = 'answer_{}'.format(i + 1)
                answer_dict[tp_key] = getattr(form, tp_key).data
            tp_choice = form.answer_10.data
            save_choice(user_id, task_id="ATI_attention", choice=tp_choice, answer_type="attention")
            if tp_choice == '6':
                session['attention_chck_right'] += 1
            else:
                session['attention_chck_fail'] += 1
                if session['attention_chck_fail'] >= 2:
                    return redirect(failure_link)
            db = get_db()
            # error = None
            from flaskr.db import ATI_PreQ
            # print(user_id)
            # preq_obj = ATI_PreQ(user_id=user_id, answer_1=answer_1, answer_2=answer_2, answer_3=answer_3, answer_4=answer_4,\
            #  answer_5=answer_5, answer_6=answer_6, answer_7=answer_7, answer_8=answer_8, answer_9=answer_9)
            q = db.query(ATI_PreQ.user_id).filter(ATI_PreQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                preq_obj = ATI_PreQ(**answer_dict)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            else:
                stmt = update(ATI_PreQ).where(ATI_PreQ.user_id==user_id).values(**answer_dict).execution_options(synchronize_session="fetch")
                result = db.execute(stmt)
                db.commit()
                current_app.logger.info("User {} ATI_preq exist, update ATI_PreQ".format(user_id))
            # current_app.logger.info("User {}'s data is saved into database, then move to the Analogy questionnaire".format(user_id))
            current_app.logger.info("User {}'s data is saved into database, then move to the first task".format(user_id))
            # return redirect(url_for('loan_analogy.Analogy_preq'))
            user_condition = session["condition"]
            if user_condition == "pilot":
                return redirect(url_for('ham_medical.ham_task_pilot'))
            return redirect(url_for('ham_medical.ham_task_base'))
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('questionnaire/ATI_pre.html', form=form)

TiA_choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')]

class TiA_Form(FlaskForm):
    # Reliability/Competence
    answer_1 = RadioField('The AI system is capable of interpreting situations correctly.', choices=TiA_choices, validators=[DataRequired()])
    answer_6 = RadioField('The AI system works reliably.', choices=TiA_choices, validators=[DataRequired()])
    answer_10 = RadioField('The AI system malfunction is likely.', choices=TiA_choices, validators=[DataRequired()])
    answer_13 = RadioField('The AI system is capable of taking over complicated tasks.', choices=TiA_choices, validators=[DataRequired()])
    answer_15 = RadioField('The AI system might make sporadic errors.', choices=TiA_choices, validators=[DataRequired()])
    answer_19 = RadioField('I am confident about the AI system’s capabilities.', choices=TiA_choices, validators=[DataRequired()])

    # Understanding/Predictability
    answer_2 = RadioField('The AI system state was always clear to me.', choices=TiA_choices, validators=[DataRequired()])
    answer_7 = RadioField('The AI system reacts unpredictably.', choices=TiA_choices, validators=[DataRequired()])
    answer_11 = RadioField('I was able to understand why things happened.', choices=TiA_choices, validators=[DataRequired()])
    answer_16 = RadioField('It’s difficult to identify what the AI system will do next.', choices=TiA_choices, validators=[DataRequired()])

    # Familiarity
    answer_3 = RadioField('I already know similar AI systems.', choices=TiA_choices, validators=[DataRequired()])
    answer_17 = RadioField('I have already used similar AI systems.', choices=TiA_choices, validators=[DataRequired()])

    # Attention check
    answer_20 = RadioField('Attention check: choose option strongly disagree.', choices=TiA_choices, validators=[DataRequired()])

    # Intention of Developers
    answer_4 = RadioField('The developers are trustworthy.', choices=TiA_choices, validators=[DataRequired()])
    answer_8 = RadioField('The developers take my well-being seriously.', choices=TiA_choices, validators=[DataRequired()])

    # Propensity to Trust
    answer_5 = RadioField('One should be careful with unfamiliar automated AI systems.', choices=TiA_choices, validators=[DataRequired()])
    answer_12 = RadioField('I rather trust an AI system than I mistrust it.', choices=TiA_choices, validators=[DataRequired()])
    answer_18 = RadioField('Automated AI systems generally work well.', choices=TiA_choices, validators=[DataRequired()])
    
    # Trust in Automation
    answer_9 = RadioField('I trust the AI system.', choices=TiA_choices, validators=[DataRequired()])
    answer_14 = RadioField('I can rely on the AI system.', choices=TiA_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

@ham_bp.route('/TiA_postq', methods=('GET', 'POST'))
def TiA_postq():
    form = TiA_Form()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in TiA post-questionnaire".format(user_id))
            answer_dict = {"user_id": user_id}
            for i in range(19):
                tp_key = 'answer_{}'.format(i + 1)
                answer_dict[tp_key] = getattr(form, tp_key).data
            tp_choice = form.answer_20.data
            save_choice(user_id, task_id="TiA_attention", choice=tp_choice, answer_type="attention")
            if tp_choice == '1':
                session['attention_chck_right'] += 1
            else:
                session['attention_chck_fail'] += 1
                if session['attention_chck_fail'] >= 2:
                    return redirect(failure_link)
            db = get_db()
            # error = None
            from flaskr.db import TiA_PostQ
            q = db.query(TiA_PostQ.user_id).filter(TiA_PostQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                # print(user_id)
                preq_obj = TiA_PostQ(**answer_dict)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            current_app.logger.info("User {}'s data is saved into database, then move to post-task questionaire".format(user_id))
            if session['condition'] == "control":
                # We do not assess learning effect in the control condition, no explanation is provided
                # return redirect(success_link)
                return redirect(url_for('ham_medical.postq_3'))
            # Except control condition, we assess the benign features, malignant features, and feedback to explanation and analogy
            return redirect(url_for('ham_medical.postq'))
            # flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('questionnaire/TiA_post.html', form=form)

# concept_list = ["Blue-Whitish Veil", "Regular Dots & Globules", "Irregular Dots & Globules", "Regression Structures",
#                 "Irregular Streaks", "Regular Streaks", "Atypical Pigment Network", "Typical Pigment Network"]

nasa_choices = [(str(x), str(x)) for x in range(-7, 8)]

class NasatlxForm(FlaskForm):
    # mental_demand = DecimalRangeField('How mentally demanding was the task?', choices=nasa_choices, validators=[DataRequired()])
    mental_demand = IntegerRangeField('1. How mentally demanding was the task?', validators=[DataRequired(), NumberRange(min=-7, max=7)], 
        render_kw={"oninput": "updateTextInput('mental_demand_value', this.value);", "onchange": "updateTextInput('mental_demand_value', this.value);"})
    physical_demand = IntegerRangeField('2. How physically demanding was the task?', validators=[DataRequired(), NumberRange(min=-7, max=7)], 
        render_kw={"oninput": "updateTextInput('physical_demand_value', this.value);", "onchange": "updateTextInput('physical_demand_value', this.value);"})
    temporal_demand = IntegerRangeField('3. How hurried or rushed was the pace of task?', validators=[DataRequired(), NumberRange(min=-7, max=7)], 
        render_kw={"oninput": "updateTextInput('temporal_demand_value', this.value);", "onchange": "updateTextInput('temporal_demand_value', this.value);"})
    performance = IntegerRangeField('4. How successful were you in accomplishing what you were asked to do?', validators=[DataRequired(), NumberRange(min=-7, max=7)], 
        render_kw={"oninput": "updateTextInput('performance_value', this.value);", "onchange": "updateTextInput('performance_value', this.value);"})
    effort = IntegerRangeField('5. How hard did you have to work to accomplish your level of performance?', validators=[DataRequired(), NumberRange(min=-7, max=7)], 
        render_kw={"oninput": "updateTextInput('effort_value', this.value);", "onchange": "updateTextInput('effort_value', this.value);"})
    frustration = IntegerRangeField('6. How insecure, discouraged, irritated, stressed, and annoyed were you?', validators=[DataRequired(), NumberRange(min=-7, max=7)], 
        render_kw={"oninput": "updateTextInput('frustration_value', this.value);", "onchange": "updateTextInput('frustration_value', this.value);"})
    submit_button = SubmitField('Submit')


@ham_bp.route('/nasatlx', methods=('GET', 'POST'))
def nasatlx():
    form = NasatlxForm()
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        db = get_db()
        error = None
        mental_demand = form.mental_demand.data
        physical_demand = form.physical_demand.data
        temporal_demand = form.temporal_demand.data
        performance = form.performance.data
        effort = form.effort.data
        frustration = form.frustration.data
        save_expertise(user_id, "mental_demand", mental_demand)
        save_expertise(user_id, "physical_demand", physical_demand)
        save_expertise(user_id, "temporal_demand", temporal_demand)
        save_expertise(user_id, "performance", performance)
        save_expertise(user_id, "effort", effort)
        save_expertise(user_id, "frustration", frustration)
        # the whole process finished and we give back completion code
        # return redirect(url_for('thanks'))
        # return render_template('thanks.html', form=form)
        return redirect(url_for('ham_medical.TiA_postq'))
        # Replace with the Prolific link

        flash(error)
    return render_template('ham_medical/nasatlx.html', form=form)

concept_choices = [('Blue-Whitish Veil', 'Blue-Whitish Veil'), ('Dots & Globules - type 1', 'Dots & Globules - type 1'),
 ('Dots & Globules - type 2', 'Dots & Globules - type 2'), ('Regression Structures', 'Regression Structures'),
 ('Streaks - type 1', 'Streaks - type 1'), ("Streaks - type 2", "Streaks - type 2"),
 ("Pigment Network - type 1", "Pigment Network - type 1"), ("Pigment Network - type 2", "Pigment Network - type 2")]


class PostQForm(FlaskForm):
    # Users' learning effect about task
    malignant_features = MultiCheckboxField('1. Please select the concepts positively associated with malignant skin lesions.', choices=concept_choices, validators=[MultiCheckboxAtLeastOne(), NoConflictChoice()])
    benign_features = MultiCheckboxField('2. Please select the concepts positively associated with benign skin lesions.', choices=concept_choices, validators=[MultiCheckboxAtLeastOne()])
    submit_button = SubmitField('Submit')


@ham_bp.route('/postq', methods=('GET', 'POST'))
def postq():
    form = PostQForm()
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        db = get_db()
        error = None
        malignant_features = "|".join(form.malignant_features.data)
        benign_features = "|".join(form.benign_features.data)
        postq_obj = PostQ_ham(user_id=user_id, malignant_features=malignant_features, benign_features=benign_features)
        q = db.query(PostQ_ham.user_id).filter(PostQ_ham.user_id==user_id)
        exist_flag = db.query(q.exists()).scalar()
        if not exist_flag:
            db.add(postq_obj)
            db.commit()
        # the whole process finished and we give back completion code
        # return redirect(url_for('thanks'))
        # return render_template('thanks.html', form=form)
        return redirect(url_for('ham_medical.postq_2'))
        # Replace with the Prolific link

        flash(error)
    return render_template('ham_medical/postq.html', form=form)

helpfulness_choices = [('unhelpful', 'unhelpful'), ('somewhat unhelpful', 'somewhat unhelpful'), ('neutral', 'neutral'),
                        ('somewhat helpful', 'somewhat helpful'), ('helpful', 'helpful')]

class PostQForm_explanation(FlaskForm):
    opinion_explanation = RadioField('To what extent do you find the explanations helpful to make decisions?', choices=helpfulness_choices, validators=[DataRequired()])
    reason_explanation = TextAreaField('Why did you find the explanation to be helpful or not helpful?', validators=[DataRequired(), Length(max=500)])
    submit_button = SubmitField('Submit')

class PostQForm_analogy(FlaskForm):
    opinion_explanation = RadioField('To what extent do you find the explanations helpful to make decisions?', choices=helpfulness_choices, validators=[DataRequired()])
    reason_explanation = TextAreaField('Why did you find the explanation to be helpful or not helpful?', validators=[DataRequired(), Length(max=500)])
    
    opinion_analogy = RadioField('To what extent do you find the analogies helpful to make decisions?', choices=helpfulness_choices, validators=[DataRequired()])
    reason_analogy = TextAreaField('Why did you find the analogies to be helpful or not helpful?', validators=[DataRequired(), Length(max=500)])
    comments_analogy = TextAreaField('Please share any comments, remarks or suggestions regarding the use of analogies to explain the medical concepts.', validators=[DataRequired(), Length(max=500)])
    submit_button = SubmitField('Submit')



@ham_bp.route('/postq_2', methods=('GET', 'POST'))
def postq_2():
    user_condition = session["condition"]
    if user_condition == "concept" or user_condition == "archie_target":
        form = PostQForm_explanation()
    else:
        #  user_condition == "archie" or user_condition == "archie_od"
        form = PostQForm_analogy()
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        db = get_db()
        error = None
        if user_condition == "concept" or user_condition == "archie_target":
            opinion_explanation = form.opinion_explanation.data
            reason_explanation = form.reason_explanation.data
            opinion_analogy = "None"
            reason_analogy = "None"
            comments_analogy = "None"
        else:
            opinion_explanation = form.opinion_explanation.data
            reason_explanation = form.reason_explanation.data
            opinion_analogy = form.opinion_analogy.data
            reason_analogy = form.reason_analogy.data
            comments_analogy = form.comments_analogy.data
        postq_obj = PostQ_ham_2(user_id=user_id, opinion_explanation=opinion_explanation, reason_explanation=reason_explanation,
            opinion_analogy=opinion_analogy, reason_analogy=reason_analogy, comments_analogy=comments_analogy)
        q = db.query(PostQ_ham_2.user_id).filter(PostQ_ham_2.user_id==user_id)
        exist_flag = db.query(q.exists()).scalar()
        if not exist_flag:
            db.add(postq_obj)
            db.commit()
        # the whole process finished and we give back completion code
        # return redirect(success_link)
        return redirect(url_for('ham_medical.postq_3'))
        # Replace with the Prolific link

        flash(error)
    return render_template('ham_medical/postq_2.html', form=form, condition=user_condition)

class PostQForm_decision(FlaskForm):
    reason_decision = TextAreaField('Please describe how you made your decisions in these tasks.', validators=[DataRequired(), Length(max=1000)])
    submit_button = SubmitField('Submit')

@ham_bp.route('/postq_3', methods=('GET', 'POST'))
def postq_3():
    user_condition = session["condition"]
    form = PostQForm_decision()
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        db = get_db()
        error = None
        reason_decision = form.reason_decision.data
        postq_obj = PostQ_ham_3(user_id=user_id, reason_decision=reason_decision)
        q = db.query(PostQ_ham_3.user_id).filter(PostQ_ham_3.user_id==user_id)
        exist_flag = db.query(q.exists()).scalar()
        if not exist_flag:
            db.add(postq_obj)
            db.commit()
        # the whole process finished and we give back completion code
        return redirect(success_link)
        # Replace with the Prolific link

        flash(error)
    return render_template('ham_medical/postq_3.html', form=form)





