import functools

from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for, current_app
)
from werkzeug.security import check_password_hash, generate_password_hash
from wtforms.validators import StopValidation
from flaskr.db import get_db
from flaskr.db import PostQ_analogy, PostQ_accuracy
from flaskr.util import vaccine_analogy, train_analogy, weather_analogy

bp = Blueprint('questionnaire', __name__, url_prefix='/questionnaire')


class OptionRequired(object):
    """
    Checks the field's data is 'truthy' otherwise stops the validation chain.

    This validator checks that the ``data`` attribute on the field is a 'true'
    value (effectively, it does ``if field.data``.) Furthermore, if the data
    is a string type, a string containing only whitespace characters is
    considered false.

    If the data is empty, also removes prior errors (such as processing errors)
    from the field.

    **NOTE** this validator used to be called `Required` but the way it behaved
    (requiring coerced data, not input data) meant it functioned in a way
    which was not symmetric to the `Optional` validator and furthermore caused
    confusion with certain fields which coerced data to 'falsey' values like
    ``0``, ``Decimal(0)``, ``time(0)`` etc. Unless a very specific reason
    exists, we recommend using the :class:`InputRequired` instead.

    :param message:
        Error message to raise in case of a validation error.
    """
    field_flags = ('required', )

    def __init__(self, message=None):
        self.message = message

    def __call__(self, form, field):
        if field.data == '0':
            if self.message is None:
                message = field.gettext('Must choose an option in this field.')
            else:
                message = self.message

            field.errors[:] = []
            raise StopValidation(message)

from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, SelectField, RadioField
from wtforms.validators import DataRequired, Length
class PreFrom(FlaskForm):
    answer_1 = RadioField('How good are you at working with fractions? -> (1: not at all good to 6: extremely good)', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    answer_2 = RadioField('How good are you at working with percentages? -> (1: not at all good to 6: extremely good)', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    # answer_2 = RadioField('How good are you at working with percentages?', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    answer_3 = RadioField('How good are you at calculating a 15% tip? -> (1: not at all good to 6: extremely good)', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    answer_4 = RadioField('How good are you at figuring out how much a shirt will cost if it is 25% off? -> (1: not at all good to 6: extremely good)', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    answer_5 = RadioField('When reading the newspaper, how helpful do you find tables and graphs that are parts of a story? -> (1 = not at all, 6 = extremely)', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    answer_6 = RadioField('When people tell you the chance of something happening, do you prefer that they use words ("it rarely happens") or numbers ("there’s a 1% chance")? -> (1 = always prefer words, 6 = always prefer numbers)', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    
    answer_7 = RadioField('When you hear a weather forecast, do you prefer predictions using percentages (e.g., "there will be a 20% chance of rain today") or predictions using only words (e.g., "there is a small chance of rain today")? -> (1 = always prefer percentages, 6 = always prefer words; reverse coded)', choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    answer_8 = RadioField("How often do you find numerical information to be useful? -> (1 = never, 6 = very often)", choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])
    answer_9 = RadioField("Attention check question: choose 3. -> (1: not at all good to 6: extremely good)", choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')], validators=[DataRequired()])

    submit_button = SubmitField('Submit')


# ATI_choices = [('completely disagree', 'completely disagree'), ('largely disagree', 'largely disagree'), ('slightly disagree', 'slightly disagree'), ('slightly agree', 'slightly agree'), ('largely agree', 'largely agree'), ('completely agree', 'completely agree')]
ATI_choices = choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5'), ('6', '6')]
class ATIForm(FlaskForm):
    answer_1 = RadioField('I like to occupy myself in greater detail with technical systems.', choices=ATI_choices, validators=[DataRequired()])
    answer_2 = RadioField('I like testing the functions of new technical systems.', choices=ATI_choices, validators=[DataRequired()])
    answer_3 = RadioField('I predominantly deal with technical systems because I have to.', choices=ATI_choices, validators=[DataRequired()])
    answer_4 = RadioField('When I have a new technical system in front of me, I try it out intensively.', choices=ATI_choices, validators=[DataRequired()])
    answer_5 = RadioField('I enjoy spending time becoming acquainted with a new technical system.', choices=ATI_choices, validators=[DataRequired()])
    answer_6 = RadioField('It is enough for me that a technical system works; I don’t care how or why.', choices=ATI_choices, validators=[DataRequired()])
    
    answer_7 = RadioField('I try to understand how a technical system exactly works.', choices=ATI_choices, validators=[DataRequired()])
    answer_8 = RadioField('It is enough for me to know the basic functions of a technical system.', choices=ATI_choices, validators=[DataRequired()])
    answer_9 = RadioField('I try to make full use of the capabilities of a technical system.', choices=ATI_choices, validators=[DataRequired()])
    answer_10 = RadioField('Attention check: choose option completely agree.', choices=ATI_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

TiA_choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')]

class TiA_Form(FlaskForm):
    # Reliability/Competence
    answer_1 = RadioField('The system is capable of interpreting situations correctly.', choices=TiA_choices, validators=[DataRequired()])
    answer_6 = RadioField('The system works reliably.', choices=TiA_choices, validators=[DataRequired()])
    answer_10 = RadioField('A system malfunction is likely.', choices=TiA_choices, validators=[DataRequired()])
    answer_13 = RadioField('The system is capable of taking over complicated tasks.', choices=TiA_choices, validators=[DataRequired()])
    answer_15 = RadioField('The system might make sporadic errors.', choices=TiA_choices, validators=[DataRequired()])
    answer_19 = RadioField('I am confident about the system’s capabilities.', choices=TiA_choices, validators=[DataRequired()])

    # Understanding/Predictability
    answer_2 = RadioField('The system state was always clear to me.', choices=TiA_choices, validators=[DataRequired()])
    answer_7 = RadioField('The system reacts unpredictably.', choices=TiA_choices, validators=[DataRequired()])
    answer_11 = RadioField('I was able to understand why things happened.', choices=TiA_choices, validators=[DataRequired()])
    answer_16 = RadioField('It’s difficult to identify what the system will do next.', choices=TiA_choices, validators=[DataRequired()])

    # Familiarity
    answer_3 = RadioField('I already know similar systems.', choices=TiA_choices, validators=[DataRequired()])
    answer_17 = RadioField('I have already used similar systems.', choices=TiA_choices, validators=[DataRequired()])

    # Attention check
    answer_20 = RadioField('Attention check: choose option strongly disagree.', choices=TiA_choices, validators=[DataRequired()])

    # Intention of Developers
    answer_4 = RadioField('The developers are trustworthy.', choices=TiA_choices, validators=[DataRequired()])
    answer_8 = RadioField('The developers take my well-being seriously.', choices=TiA_choices, validators=[DataRequired()])

    # Propensity to Trust
    answer_5 = RadioField('One should be careful with unfamiliar automated systems.', choices=TiA_choices, validators=[DataRequired()])
    answer_12 = RadioField('I rather trust a system than I mistrust it.', choices=TiA_choices, validators=[DataRequired()])
    answer_18 = RadioField('Automated systems generally work well.', choices=TiA_choices, validators=[DataRequired()])
    
    # Trust in Automation
    answer_9 = RadioField('I trust the system.', choices=TiA_choices, validators=[DataRequired()])
    answer_14 = RadioField('I can rely on the system.', choices=TiA_choices, validators=[DataRequired()])

    submit_button = SubmitField('Submit')

class PostFrom(FlaskForm):
    answer_1 = StringField('1 + 1 = ', validators=[DataRequired()])
    answer_2 = StringField('2 + 2 = ', validators=[DataRequired()])
    answer_3 = StringField('3 + 3 = ', validators=[DataRequired()])
    answer_4 = StringField('4 + 4 = ', validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class PostFrom_accuracy(FlaskForm):
    answer_1 = RadioField('How much did you rely on the system accuracy in deciding whether to follow the system’s advice? -> (1: not at all to 5: very much)', choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')], validators=[DataRequired()])
    submit_button = SubmitField('Submit')

class PostFrom_analogy_vaccine(FlaskForm):
    answer_1 = RadioField('How helpful did you find the analogy? -> (1: not at all helpful to 5: very helpful)', choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')], validators=[DataRequired()])
    answer_2 = RadioField('How familiar were you with the chosen analogy: vaccine efficacy? -> (1: unfamiliar to 5: very familiar)', choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')], validators=[DataRequired()])
    answer_3 = StringField('Why did you find the analogy to be helpful or not helpful?', validators=[DataRequired(), Length(max=500)])
    answer_4 = StringField('Please share any comments, remarks or suggestions regarding the use of analogies to explain the accuracy of the system.', validators=[DataRequired(), Length(max=500)])
    submit_button = SubmitField('Submit')

class PostFrom_analogy_train(FlaskForm):
    answer_1 = RadioField('How helpful did you find the analogy? -> (1: not at all helpful to 5: very helpful)', choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')], validators=[DataRequired()])
    answer_2 = RadioField('How familiar were you with the chosen analogy: train punctuality? -> (1: unfamiliar to 5: very familiar)', choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')], validators=[DataRequired()])
    answer_3 = StringField('Why did you find the analogy to be helpful or not helpful?', validators=[DataRequired(), Length(max=500)])
    answer_4 = StringField('Please share any comments, remarks or suggestions regarding the use of analogies to explain the accuracy of the system.', validators=[DataRequired(), Length(max=500)])
    submit_button = SubmitField('Submit')

class PostFrom_analogy_weather(FlaskForm):
    answer_1 = RadioField('How helpful did you find the analogy? -> (1: not at all helpful to 5: very helpful)', choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')], validators=[DataRequired()])
    answer_2 = RadioField('How familiar were you with the chosen analogy: weather report accuracy? -> (1: unfamiliar to 5: very familiar)', choices = [('1', '1'), ('2', '2'), ('3', '3'), ('4', '4'), ('5', '5')], validators=[DataRequired()])
    answer_3 = StringField('Why did you find the analogy to be helpful or not helpful?', validators=[DataRequired(), Length(max=500)])
    answer_4 = StringField('Please share any comments, remarks or suggestions regarding the use of analogies to explain the accuracy of the system.', validators=[DataRequired(), Length(max=500)])
    submit_button = SubmitField('Submit')

def flash_errors(form):
    """Flashes form errors"""
    for field, errors in form.errors.items():
        for error in errors:
            flash(u"Error in the %s field - %s" % (
                getattr(form, field).label.text,
                error
            ), 'error')

def save_choice(user_id, task_id, choice, answer_type="attention"):
    db = get_db()
    error = None
    from flaskr.db import UserTask
    q = db.query(UserTask.user_id).filter(UserTask.user_id==user_id, UserTask.task_id==task_id, UserTask.answer_type==answer_type)
    exist_flag = db.query(q.exists()).scalar()
    if not exist_flag:
        ut_obj = UserTask(user_id=user_id, task_id=task_id, choice=choice, answer_type=answer_type)
        db.add(ut_obj)
        db.commit()

@bp.route('/ATI_preq', methods=('GET', 'POST'))
def ATI_preq():
    form = ATIForm()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in pre-questionnaire".format(user_id))
            # answer_1 = form.answer_1.data
            # answer_2 = form.answer_2.data
            # answer_3 = form.answer_3.data
            # answer_4 = form.answer_4.data
            # answer_5 = form.answer_5.data
            # answer_6 = form.answer_6.data
            # answer_7 = form.answer_7.data
            # answer_8 = form.answer_8.data
            # answer_9 = form.answer_9.data
            answer_dict = {"user_id": user_id}
            for i in range(9):
                tp_key = 'answer_{}'.format(i + 1)
                answer_dict[tp_key] = getattr(form, tp_key).data
            tp_choice = form.answer_10.data
            save_choice(user_id, task_id="ATI_attention", choice=tp_choice, answer_type="attention")
            if tp_choice == '6':
                session['attention_chck_right'] += 1
            db = get_db()
            error = None
            from flaskr.db import ATI_PreQ
            # print(user_id)
            # preq_obj = ATI_PreQ(user_id=user_id, answer_1=answer_1, answer_2=answer_2, answer_3=answer_3, answer_4=answer_4,\
            #  answer_5=answer_5, answer_6=answer_6, answer_7=answer_7, answer_8=answer_8, answer_9=answer_9)
            q = db.query(ATI_PreQ.user_id).filter(ATI_PreQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                preq_obj = ATI_PreQ(**answer_dict)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            current_app.logger.info("User {}'s data is saved into database, then move to the first example".format(user_id))
            return redirect(url_for('loan_task.example_1'))

            flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('questionnaire/ATI_pre.html', form=form)

@bp.route('/preq', methods=('GET', 'POST'))
def preq():
    form = PreFrom()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in pre-questionnaire".format(user_id))
            # answer_1 = request.form['answer_1']
            # answer_2 = request.form['answer_2']
            # answer_3 = request.form['answer_3']
            # answer_4 = request.form['answer_4']
            answer_1 = form.answer_1.data
            answer_2 = form.answer_2.data
            answer_3 = form.answer_3.data
            answer_4 = form.answer_4.data
            answer_5 = form.answer_5.data
            answer_6 = form.answer_6.data
            answer_7 = form.answer_7.data
            answer_8 = form.answer_8.data
            tp_choice = form.answer_9.data
            save_choice(user_id, task_id="Numeracy Test", choice=tp_choice, answer_type="attention")
            if tp_choice == '3':
                session['attention_chck_right'] += 1
            db = get_db()
            error = None
            from flaskr.db import PreQ
            q = db.query(PreQ.user_id).filter(PreQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                # print(user_id)
                preq_obj = PreQ(user_id=user_id, answer_1=answer_1, answer_2=answer_2, answer_3=answer_3, answer_4=answer_4,\
                 answer_5=answer_5, answer_6=answer_6, answer_7=answer_7, answer_8=answer_8)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            current_app.logger.info("User {}'s data is saved into database, then move to ATI questionnaire".format(user_id))
            return redirect(url_for('questionnaire.ATI_preq'))

            flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('questionnaire/pre.html', form=form)

@bp.route('/postq', methods=('GET', 'POST'))
def postq():
    # to-do: different group of participants may see different questionnaires
    # if session['attention_chck_right'] > 0:
    #     at least one attention check is passed
    #     return redirect("https://app.prolific.co/submissions/complete?cc=61FE0119")
    if session['attention_chck_right'] == 0:
        # at least one attention check is passed
        return render_template('thanks.html', reason_str="Unfortunately, you have failed both attention checks and will not be allowed to proceed any further.")
    user_group = session['user_group']
    analogy_type = session['analogy_type']
    if user_group == "analogy":
        if analogy_type == "vaccine":
            form = PostFrom_analogy_vaccine()
        elif analogy_type == "train":
            form = PostFrom_analogy_train()
        else:
            form = PostFrom_analogy_weather()
    elif user_group == "accuracy":
        form = PostFrom_accuracy()
    else:
        # we don't have other questions for pilot study and group 1
        return redirect(url_for('loan_task.result'))
    if request.method == 'POST' and form.validate_on_submit():
        user_id = session['user_id']
        db = get_db()
        error = None
        if user_group == "analogy":
            answer_1 = form.answer_1.data
            answer_2 = form.answer_2.data
            answer_3 = form.answer_3.data
            answer_4 = form.answer_4.data
            postq_obj = PostQ_analogy(user_id=user_id, answer_1=answer_1, answer_2=answer_2, answer_3=answer_3, answer_4=answer_4)
            q = db.query(PostQ_analogy.user_id).filter(PostQ_analogy.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
        else:
            # user_group = "accuracy"
            answer_1 = form.answer_1.data
            postq_obj = PostQ_accuracy(user_id=user_id, answer_1=answer_1)
            q = db.query(PostQ_accuracy.user_id).filter(PostQ_accuracy.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
        if not exist_flag:
            db.add(postq_obj)
            db.commit()
        # the whole process finished and we give back completion code
        # return redirect(url_for('thanks'))
        # return render_template('thanks.html', form=form)
        return redirect(url_for('loan_task.result'))

        flash(error)
    # deliver pre-task questionnaire page
    if user_group == "analogy":
        if analogy_type == "vaccine":
            analogy_str = vaccine_analogy
        elif analogy_type == "train":
            analogy_str = train_analogy
        else:
            analogy_str = weather_analogy
        return render_template('questionnaire/post_analogy.html', form=form, analogy_str=analogy_str)
    elif user_group == "accuracy":
        return render_template('questionnaire/post_accuracy.html', form=form)
    else:
        return redirect(url_for('loan_task.result'))


@bp.route('/TiA_postq', methods=('GET', 'POST'))
def TiA_postq():
    form = TiA_Form()
    if request.method == 'POST':
        # print("PreFrom submitted through POST.")
        if form.validate_on_submit():
            # print("PreFrom submitted successfully.")
            user_id = session['user_id']
            current_app.logger.info("User {} filled in TiA post-questionnaire".format(user_id))
            answer_dict = {"user_id": user_id}
            for i in range(19):
                tp_key = 'answer_{}'.format(i + 1)
                answer_dict[tp_key] = getattr(form, tp_key).data
            tp_choice = form.answer_20.data
            save_choice(user_id, task_id="TiA_attention", choice=tp_choice, answer_type="attention")
            if tp_choice == '1':
                session['attention_chck_right'] += 1
            db = get_db()
            error = None
            from flaskr.db import TiA_PostQ
            q = db.query(TiA_PostQ.user_id).filter(TiA_PostQ.user_id==user_id)
            exist_flag = db.query(q.exists()).scalar()
            if not exist_flag:
                # print(user_id)
                preq_obj = TiA_PostQ(**answer_dict)
                # current_app.logger.info("User {}'s data is recorded into preq-obj".format(user_id))
                db.add(preq_obj)
                db.commit()
            current_app.logger.info("User {}'s data is saved into database, then move to ATI questionnaire".format(user_id))
            return redirect(url_for('questionnaire.postq'))

            flash(error)
        else:
            # flash_errors(form)
            print("Not validate_on_submit.")
            print(form.errors)
    # deliver pre-task questionnaire page
    # return render_template('questionnaire/pre_new.html')
    return render_template('questionnaire/TiA_post.html', form=form)


if __name__ == '__main__':
    from sqlalchemy import create_engine
    from sqlalchemy_utils import database_exists, create_database
    from sqlalchemy.orm import sessionmaker
    user_id = "root"
    user_password = "hegaole945"
    dbname = "Analogy"

    # Create engine through sqlalchemy
    # engine = create_engine("mysql+pymysql://%s:%s@localhost:3306/%s" %(user_id,user_password,dbname),echo=False)
    engine = create_engine("mysql+pymysql://%s:%s@localhost:3306/%s" %(user_id,user_password,dbname),echo=False)

    # Create database if not exists
    if not database_exists(engine.url): 
        create_database(engine.url)

    DBSession = sessionmaker(bind=engine)
    db = DBSession()
    error = None
    from flaskr.db import PreQ
    # print(user_id)
    preq_obj = PreQ(user_id="gaole", answer_1="sad", answer_2='no', answer_3='asd', answer_4='asd',\
     answer_5='asd', answer_6='asd', answer_7='answer_7', answer_8='answer_8')
    db.add(preq_obj)
    db.commit()
    db.close()

